#!/bin/bash
# ./chkexpjlt.sh arch expid sesth >& expid.log &

#set -xuve

arch=$1
expid=$2
sesth=$3 # sim (job) estimated computing time (in hours)
src=/cfu/autosubmit/$expid/tmp
case $arch in
 mn-*)     rsrc=/gpfs/scratch/*/*; rptr="err" ;;
 ithaca)   rsrc=/scratch/cfu/*; rptr="\.e" ;;
 hector)   rsrc= ; rptr="" ;;
 lindgren) rsrc=/cfs/klemming/scratch/*/* ;;
 jaguar)   rsrc= ; rptr="" ;;
 *) echo "!!! $arch is not available !!!"; exit 1 ;;
esac
rlst=/tmp/$expid.chkexpjlt.$$
ssh $arch ls -1 $rsrc/$expid/LOG_$expid/* > $rlst

#compute job life time (jlt) for sim jobs
echo "Job_Number Job_Life_Time(Hours) Job_Wasted_Time(Hours) Job_Failed Remarks"
typ=sim
cnt=0
sdates=$(ls -1 $src/${expid}*${typ}_COMPLETED | cut -d '_' -f 2 | uniq)
started=$(ls -lrth $src/*.cmd | head -n 1 | awk '{print $6 " " $7 " " $8}')
for s in $sdates; do 
 mems=$(ls -1 $src/${expid}_${s}_*_${typ}_COMPLETED | cut -d '_' -f 3 | uniq)
 for m in $mems; do
  chunks=$(ls -1 $src/${expid}_${s}_${m}_*_${typ}_COMPLETED | wc -l)
  for c in $(seq 1 $chunks); do
   cnt=$((cnt+1))
   job=${expid}_${s}_${m}_${c}_${typ}
   case $c in
    1) job_1=${expid}_${s}_${m}_${c}_init ;;
    *) job_1=${expid}_${s}_${m}_$((c-1))_${typ} ;;
   esac
   ts=$(ls -l $src/${job_1}_COMPLETED | awk '{print $6 " " $7 " " $8}'); ts=$(date --date="$ts" +%s)
   te=$(ls -l $src/${job}_COMPLETED | awk '{print $6 " " $7 " " $8}'); te=$(date --date="$te" +%s)
   let "jlt=$te-$ts"
   jlt=$(echo | awk "{print $jlt/3600}"); njlt=$(printf "%0.2f" $jlt)
   nfj=$(cat $rlst | grep ${job} | grep $rptr | wc -l)
   jwt=$(echo | awk "{print $jlt-$sesth}"); njwt=$(printf "%0.2f" $jwt) # job wasted time
   echo $cnt $njlt $njwt $((nfj-1)) $job
  done
 done
 #echo
done
till=$(ls -lrth $src/*COMPLETED | tail -n 1 | awk '{print $6 " " $7 " " $8}')
echo "$expid (optimum computing time per job ~${sesth}H)"
echo "started: $started - till: $till"
rm -f $rlst

